package com.jwipc.sys.convert;

import com.jwipc.sys.dto.BusinessAttachmentDTO;
import com.jwipc.sys.entity.BusinessAttachment;
import com.jwipc.sys.entity.DigitalSealBusinessDetail;
import com.jwipc.sys.model.po.BusinessAttachmentPO;
import org.mapstruct.Mapper;

import java.util.List;

/**
 * <p>
 *
 * </p>
 *
 * @author Chen Tianshi
 * @since 2025/9/12 13:42
 */

@Mapper(componentModel = "spring")
public interface BusinessAttachmentConvert {
    List<BusinessAttachment> dtoList2EntityList(List<BusinessAttachmentDTO> list);

    List<BusinessAttachmentDTO> entityList2dtoList(List<BusinessAttachment> list);

    List<BusinessAttachmentDTO> poList2dtoList(List<BusinessAttachmentPO> list);

    BusinessAttachmentPO signDetail2PO(DigitalSealBusinessDetail detail);
}
